/**
 * Copyright 2016 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
**/

'use strict';

const fs = require('fs');
const path = require('path');
const updateNotifier = require('update-notifier');
const swBuild = require('workbox-build');

const cliLogHelper = require('./lib/log-helper');
const generateGlobPattern = require('./lib/utils/generate-glob-pattern');
const saveConfigFile = require('./lib/utils/save-config');
const getConfigFile = require('./lib/utils/get-config');

const askForRootOfWebApp = require('./lib/questions/ask-root-of-web-app');
const askForServiceWorkerName = require('./lib/questions/ask-sw-name');
const askSaveConfigFile = require('./lib/questions/ask-save-config');
const askManifestFileName = require('./lib/questions/ask-manifest-name');
const askForExtensionsToCache =
  require('./lib/questions/ask-extensions-to-cache');

/**
 * This class is a wrapper to make test easier. This is used by
 * ./bin/index.js to pass in the args when the CLI is used.
 */
class SWCli {
  /**
   * This is a helper method that allows the test framework to call argv with
   * arguments without worrying about running as an actual CLI.
   *
   * @private
   * @param {Object} meowOutput The value passed in via process.argv.
   * @return {Promise} Promise is returned so testing framework knows when
   * handling the request has finished.
   */
  argv(meowOutput) {
    updateNotifier({pkg: meowOutput.pkg}).notify();

    if (meowOutput.input.length > 0) {
      // We have a command
      return this.handleCommand(
        meowOutput.input[0],
        meowOutput.input.splice(1),
        meowOutput.flags
      )
      .then(() => {
        process.exit(0);
      })
      .catch(() => {
        process.exit(1);
      });
    } else {
      meowOutput.showHelp(1);
    }
  }

  /**
   * Prints the help text to the terminal.
   * @return {string} The log output
   */
  getHelpText() {
    return fs.readFileSync(
      path.join(__dirname, 'cli-help.txt'), 'utf8');
  }

  /**
   * If a command is given in the command line args, this method will handle
   * the appropriate action.
   * @param {string} command The command name.
   * @param {object} args The arguments given to this command.
   * @param {object} flags The flags supplied with the command line.
   * @return {Promise} A promise for the provided task.
   */
  handleCommand(command, args, flags) {
    switch (command) {
      case 'generate:sw':
        return this._generateSW();
      case 'generate:manifest':
        return this._generateBuildManifest();
      default:
        cliLogHelper.error(`Invlaid command given '${command}'`);
        return Promise.reject();
    }
  }

  /**
   * This method will generate a working service worker with a file manifest.
   * @return {Promise} The promise returned here will be used to exit the
   * node process cleanly or not.
   */
  _generateSW() {
    let config = {};

    return getConfigFile()
    .then((savedConfig) => {
      if (savedConfig) {
        config = savedConfig;
        config.wasSaved = true;
      }
    })
    .then(() => {
      if (!config.globDirectory) {
        return askForRootOfWebApp()
        .then((rDirectory) => {
          // This will give a pretty relative path:
          // '' => './'
          // 'build' => './build/'
          config.globDirectory =
            path.join('.', path.relative(process.cwd(), rDirectory), path.sep);
        });
      }
    })
    .then(() => {
      if (!config.staticFileGlobs) {
        return askForExtensionsToCache(config.globDirectory)
        .then((extensionsToCache) => {
          config.staticFileGlobs = [
            generateGlobPattern(extensionsToCache),
          ];
        });
      }
    })
    .then(() => {
      if (!config.swDest) {
        return askForServiceWorkerName()
        .then((swName) => {
          config.swDest = path.join(config.globDirectory, swName);
          config.globIgnores = [
            swName,
          ];
        });
      }
    })
    .then(() => {
      if (!config.wasSaved) {
        return askSaveConfigFile();
      }
      // False since it's already saved.
      return false;
    })
    .then((saveConfig) => {
      if (saveConfig) {
        return saveConfigFile(config);
      }
    })
    .then(() => {
      return swBuild.generateSW(config);
    });
  }

  /**
   * Generates a file manifest with revisioning details
   * that can be used in your service worker for precaching assets.
   * @return {Promise} Resolves when the node process exits.
   */
  _generateBuildManifest() {
    let rootDirPath;
    let fileManifestName;
    let fileExtentionsToCache;

    return askForRootOfWebApp()
    .then((rDirectory) => {
      rootDirPath = rDirectory;
      return askForExtensionsToCache(rootDirPath);
    })
    .then((extensionsToCache) => {
      fileExtentionsToCache = extensionsToCache;
      return askManifestFileName();
    })
    .then((manifestName) => {
      fileManifestName = manifestName;
    })
    .then(() => {
      const globPattern = generateGlobPattern(fileExtentionsToCache);
      return swBuild.generateFileManifest({
        globDirectory: rootDirPath,
        staticFileGlobs: [globPattern],
        globIgnores: [
          fileManifestName,
        ],
        manifestDest: path.join(rootDirPath, fileManifestName),
      });
    });
  }
}

module.exports = SWCli;
